from django.db import models
from dj_ansible import const

"""
Bases
"""


class BaseAnsibleGroup(models.Model):
    name = models.CharField(max_length=100)
    ansible_connection = models.CharField(choices=const.ANSIBLE_CONNECTION_CHOICES, max_length=100)

    class Meta:
        abstract = True


class BaseAnsibleDeviceGroup(BaseAnsibleGroup):
    ansible_become = models.BooleanField(default=False)

    class Meta:
        abstract = True


class BaseAnsibleHost(models.Model):
    host = models.CharField(max_length=100)
    ansible_ssh_host = models.GenericIPAddressField()
    ansible_user = models.CharField(max_length=100)
    ansible_ssh_pass = models.CharField(max_length=100)

    class Meta:
        abstract = True


"""
Groups Models
"""


class AnsibleNetworkGroup(BaseAnsibleDeviceGroup):
    ansible_network_os = models.CharField(choices=const.ANSIBLE_NETWORK_OS_CHOICES, max_length=100)
    parent_group = models.ForeignKey('self', on_delete=models.DO_NOTHING, related_name='child_group',
                                     null=True, blank=True)

    class Meta:
        db_table = 'ansible_network_groups'
        app_label = 'dj_ansible'


class AnsibleServerGroup(BaseAnsibleDeviceGroup):
    parent_group = models.ForeignKey('self', on_delete=models.DO_NOTHING, related_name='child_group',
                                     null=True, blank=True)

    class Meta:
        db_table = 'ansible_server_groups'
        app_label = 'dj_ansible'


class AnsibleAWSGroup(BaseAnsibleGroup):
    ami = models.CharField(max_length=100)
    region = models.CharField(max_length=100)
    type = models.CharField(max_length=100)
    sshkey = models.CharField(max_length=100)
    vpcid = models.CharField(max_length=100)

    class Meta:
        db_table = 'ansible_aws_groups'
        app_label = 'dj_ansible'


"""
Hosts Models
"""


class AnsibleNetworkHost(BaseAnsibleHost):
    ansible_become_pass = models.CharField(max_length=100)
    group = models.ForeignKey(AnsibleNetworkGroup, on_delete=models.DO_NOTHING)

    class Meta:
        db_table = 'ansible_network_hosts'
        app_label = 'dj_ansible'


class AnsibleServerHost(BaseAnsibleHost):
    ansible_become_pass = models.CharField(max_length=100)
    group = models.ForeignKey(AnsibleServerGroup, on_delete=models.DO_NOTHING)

    class Meta:
        db_table = 'ansible_server_hosts'
        app_label = 'dj_ansible'



from ansible.plugins.callback import CallbackBase
from collections import defaultdict


class ResultCallback(CallbackBase):
    """callback plugin used for collecting play results"""

    def __init__(self):
        self.results = defaultdict(list)
        self._host_tasts_dict = defaultdict(list)
        super(ResultCallback).__init__()

    def _is_host_in_result(self, host, key):
        for item in self.results[key]:
            if host in item.values():
                return True

    def _add_callback_result_to_results(self, result, status):
        host = result._host.name
        task_name = result.task_name
        task = {
            "name": task_name,
            "result": result._result
        }
        if not self._is_host_in_result(host, key='failed'):
            self.results[status].append({
                "host": host,
                "tasks": self._host_tasts_dict[host]
            })
        self._host_tasts_dict[host].append(task)

    def v2_runner_on_failed(self, result, ignore_errors=False):
        """method to execute on failed execution"""
        self._add_callback_result_to_results(result, 'failed')
        self.custom_runner_on_failed(result, ignore_errors)

    def v2_runner_on_ok(self, result, **kwargs):
        """method to execute on Ok execution"""
        self._add_callback_result_to_results(result, 'success')
        self.custom_runner_on_ok(result, **kwargs)

    def custom_runner_on_failed(self, result, ignore_errors=False):
        pass

    def custom_runner_on_ok(self, result, **kwargs):
        pass



from ansible.parsing.dataloader import DataLoader
from ansible.inventory.manager import InventoryManager
from dj_ansible.models import AnsibleNetworkGroup, AnsibleNetworkHost
from dj_ansible.exceptions import DjangoAnsibleError


class _Inventory(InventoryManager):
    """Collect Inventory from DB"""

    def __init__(self):
        loader = DataLoader()
        self.host_model = AnsibleNetworkHost
        self.network_group_model = AnsibleNetworkGroup
        super(_Inventory, self).__init__(loader)

    def _add_group_childes(self, group):
        if group.parent_group:
            if group.parent_group.name not in self._inventory.groups:
                self._inventory.add_group(group.parent_group.name)
            self._inventory.add_child(group.parent_group.name, group.name)
            self._add_group_childes(group.parent_group)

    def _parse_network_groups(self, groups):
        try:
            for group in groups:
                self._inventory.add_group(group.name)

                for field in group._meta.fields:
                    field_name = field.name
                    if field_name == 'parent_group':
                        continue
                    value = getattr(group, field.name)
                    if value:
                        self._inventory.set_variable(group.name, field_name, value)
                self._add_group_childes(group)
            hosts = self.host_model.objects.all()
            for host in hosts:
                self._inventory.add_host(host.host, group=host.group.name)

                for field in host._meta.fields:
                    field_name = field.name
                    value = getattr(host, field.name)
                    if value:
                        self._inventory.set_variable(host.host, field_name, value)
        except Exception as e:
            raise DjangoAnsibleError(str(e))

    def parse_sources(self, cache=False):
        groups = self.network_group_model.objects.all()
        self._parse_network_groups(groups)


def get_inventory():
    inventory = _Inventory()

    return inventory



ANSIBLE_CONNECTION_CHOICES = (
    ('network_cli', 'network_cli'),
    ('paramiko_ssh', 'paramiko'),
    ('chroot', 'chroot'),
    ('docker', 'docker'),
    ('httpapi', 'httpapi'),
    ('netconf', 'netconf'),
    ('winrm', 'winrm')
)

ANSIBLE_NETWORK_OS_CHOICES = (
    ('a10', 'a10'),
    ('aci', 'aci'),
    ('aireos', 'aireos'),
    ('aos', 'aos'),
    ('aruba', 'aruba'),
    ('asa', 'asa'),
    ('f5', 'f5'),
    ('ios', 'ios'),
    ('iosxr', 'iosxr'),
    ('nxos', 'nxos')

)



import os
from setuptools import find_packages, setup
from dj_ansible import __version__, __author__, __license__

with open("README.md", "r") as fh:
    long_description = fh.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-ansible',
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    install_requires=['ansible'],
    license=__license__,
    description='Django app which provide an APIs to integrate Ansible with Django projects',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=__author__,
    author_email='aaqrabaw@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
